package com.pcloud.book.copyright.biz.impl;

import com.pcloud.book.copyright.biz.BookAuthServeBiz;
import com.pcloud.book.copyright.dao.BookAuthServeDao;
import com.pcloud.book.copyright.dto.ServeDTO;
import com.pcloud.book.copyright.entity.BookAuthServe;
import com.pcloud.book.copyright.vo.ServeVO;
import com.pcloud.channelcenter.base.exceptions.ChannelBizException;
import com.pcloud.common.core.aspect.ParamLog;
import com.pcloud.common.utils.ListUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author lily
 * @date 2018/12/3 12:02
 */
@Component("bookAuthServeBiz")
public class BookAuthServeBizImpl implements BookAuthServeBiz {
    @Autowired
    private BookAuthServeDao bookAuthServeDao;

    @Override
    @ParamLog("设置图书授权服务")
    public void setBookAuthServes(List<ServeVO> serves, Long bookId, Long channelId, Long adviserId) {
          if(bookId == null || channelId == null || adviserId == null) {
            throw new ChannelBizException(ChannelBizException._ERROR, "图书不存在");
          }
          deleteServesByBook(bookId, channelId, adviserId);
          if(!ListUtils.isEmpty(serves)){
              List<BookAuthServe> bookAuthServes = changeToBookAuthServe(serves, bookId, channelId, adviserId);
              bookAuthServeDao.insert(bookAuthServes);
          }
    }

    /**
     * 是否设置应用授权  return   key : typeCode_serveId
     */
    @ParamLog("是否设置应用授权")
    public Map<String, Boolean> isSetServeAuth(Long bookId, Long channelId, Long adviserId, List<Long> serveIds) {
        if (bookId == null || channelId == null || adviserId == null || ListUtils.isEmpty(serveIds)) {
            return null;
        }
        List<ServeDTO> serveDTOS = bookAuthServeDao.isSetServeAuth(bookId, channelId, adviserId, serveIds);
        if (ListUtils.isEmpty(serveDTOS)) {
            return null;
        }
        Map<String, Boolean> isSetServesMap = new HashMap<>();
        for (ServeDTO serveDTO : serveDTOS) {
            isSetServesMap.put(serveDTO.getServeType() + serveDTO.getServeId(), true);
        }
        return isSetServesMap;
    }

    @Override
    public List<ServeVO> listServesByBook(Long bookId, Long channelId, Long adviserId) {
       return bookAuthServeDao.listServesByBook(bookId, channelId, adviserId);
    }

    @Override
    public Boolean checkIsNeedAuth(Long bookId, Long channelId, Long adviserId, Long serveId, String serveType) {
        return bookAuthServeDao.checkIsNeedAuth(bookId, channelId, adviserId,serveId,serveType);
    }


    private List<BookAuthServe> changeToBookAuthServe(List<ServeVO> serves, Long bookId, Long channelId, Long adviserId) {
        List<BookAuthServe> bookAuthServes = new ArrayList<>();
        for (ServeVO serveVO : serves){
            BookAuthServe bookAuthServe = new BookAuthServe();
            bookAuthServe.setBookId(bookId);
            bookAuthServe.setChannelId(channelId);
            bookAuthServe.setAdviserId(adviserId);
            BeanUtils.copyProperties(serveVO, bookAuthServe);
            bookAuthServes.add(bookAuthServe);
        }
        return bookAuthServes;
    }

    @ParamLog("删除图书授权服务")
    private void deleteServesByBook(Long bookId, Long channelId, Long adviserId){
        if (bookId == null || channelId == null || adviserId == null) {
            throw new ChannelBizException(ChannelBizException._ERROR, "图书不存在");
        }
        List<Long> ids = bookAuthServeDao.selectNeedDelete(bookId, channelId, adviserId);
        if(ListUtils.isEmpty(ids)) return;
        bookAuthServeDao.deleteServes(ids);
    }

}
